
package com.allaire.cfx ;


/**
* Interface to a query used or created by a <code>{@link CustomTag}</code>.
*
* A query contains tabular data organized by named columns and rows.
*
*  @author J.J. Allaire
*/
public interface Query
{
	/**
	* Retrieves the name of the query.
	*
	* <P>The following example retrieves the name of the query
	* and writes it back to the user: </P>
	*
	* <pre><code>   Query query = request.getQuery() ;
	*   response.write( &quot;The query name is: &quot; + query.getName() ) ; </code></pre>
	*
	* @return The name of the query.
	*/
	public String getName() ;


	/**
	* Retrieves the number of rows in the query.
	*
	* <P>The following example retrieves the number of rows
	* in a query and writes it back to the user: </P>
	*
	* <pre><code>   Query query = request.getQuery() ;
	*   int rows = query.getRowCount() ;
	*   response.write( &quot;The number of rows in the query is &quot;
	*                      + Integer.toString(rows) ) ; </code></pre>
	*
	* @return The number of rows contained in the query.
	*/
	public int getRowCount() ;


	/**
	* Retrieves the index of a column given its name.
	*
	* <P>The following example retrieves the index of the EMAIL
	* column and uses it to output a list of the addresses
	* contained in the column:</P>
	*
	* <pre><code>   // Get the index of the EMAIL column
	*   int iEMail = query.getColumnIndex( &quot;EMAIL&quot; ) ;
	*
	*   // Iterate over the query and output list of addresses
	*   int nRows = query.getRowCount() ;
	*   for( int iRow=1; iRow&lt;=nRows; iRow++ )
	*   {
	*      response.write( query.getData( iRow, iEMail ) + &quot;&lt;BR&gt;&quot; ) ;
	*   } </code></pre>
	*
	* @param name name of column to get index of (lookup is case insensitive)
	*
	* @return The index of the column (returns -1 if no such column exists).
	*
	* @see #getColumns getColumns
	* @see #getData    getData
	*/
	public int getColumnIndex( String name ) ;

	/**
	* Retrieves a list of the column names contained in the query.
	*
	* <P>The following example retrieves the array of columns and then
	* iterates over the list, writing each column name back to the user: </P>
	*
	* <pre><code>   // Get the list of columns from the query
	*   String[] columns = query.getColumns() ;
	*   int nNumColumns = columns.length ;
	*
	*   // Print the list of columns to the user
	*   response.write( &quot;Columns in query: &quot; ) ;
	*   for( int i=0; i&lt;nNumColumns; i++ )
	*   {
	*      response.write( columns[i] + &quot; &quot; ) ;
	*   } </code></pre>
	*
	* @return  An array of strings containing the names of the columns in the query.
	*/
	public String[] getColumns() ;


	/**
	* Retrieves a data element from a row and column of the query.
	*
	* Row and column indexes begin with 1. You can determine the number of rows in the
	* query by calling <code>{@link #getRowCount getRowCount}</code>. You can determine the
	* columns in the query by calling <code>{@link #getColumns getColumns}</code>.
	*
	* <P>The following example iterates over the rows of the query and writes the data back to
	* the user in a simple, space-delimited format: </P>
	*
	* <pre><code>   int iRow, iCol ;
	*   int nNumCols = query.getColumns().length ;
	*   int nNumRows = query.getRowCount() ;
	*   for ( iRow=1; iRow<=nNumRows; iRow++ )
	*   {
	*      for ( iCol=1; iCol<=nNumCols; iCol++ )
	*      {
	*         response.write( query.getData( iRow, iCol ) + &quot; &quot; ) ;
	*      }
	*      response.write( &quot;&lt;BR&gt;&quot; ) ;
	*   } </code></pre>
	*
	* @param iRow row to retrieve data from (1-based)
	* @param iCol column to retrieve data from (1-based)
	*
	* @return The value of the requested data element.
	*
	* @throws <code>IndexOutOfBoundsException</code> If an invalid index is passed to the method.
	*
	* @see #setData setData
	* @see #addRow addRow
	*/
	public String getData( int iRow, int iCol ) throws IndexOutOfBoundsException ;


	/**
	* Adds a new row to the query. Call this method each
	* time you want to append a row to the query.
	*
	* <P>The following example demonstrates the addition of 2 rows
	* to a query that has 3 columns ('City', 'State', and 'Zip'): <P>
	*
	* <pre><code>   // Define column indexes
	*   int iCity = 1, iState = 2, iZip = 3 ;
	*
	*   // First row
	*   int iRow = query.addRow() ;
	*   query.setData( iRow, iCity, &quot;Minneapolis&quot; ) ;
	*   query.setData( iRow, iState, &quot;MN&quot; ) ;
	*   query.setData( iRow, iZip, &quot;55345&quot; ) ;
	*
	*   // Second row
	*   iRow = query.addRow() ;
	*   query.setData( iRow, iCity, &quot;St. Paul&quot; ) ;
	*   query.setData( iRow, iState, &quot;MN&quot; ) ;
	*   query.setData( iRow, iZip, &quot;55105&quot; ) ; </code></pre>
	*
	* @return The index of the row that was appended to the query.
	*
	* @see #setData setData
	* @see #getData getData
	*/
	public int addRow() ;

	/**
	* Sets a data element within a row and column of the query.
	* Row and column indexes begin with 1. Before calling <code>setData</code>
	* for a given row you should be sure to call <code>{@link #addRow addRow}</code>
	* and use the return value as the row index for your call to <code>setData</code>.
	*
	* <P>The following example demonstrates the addition of 2 rows
	* to a query that has 3 columns ('City', 'State', and 'Zip'): <P>
	*
	* <pre><code>   // Define column indexes
	*   int iCity = 1, iState = 2, iZip = 3 ;
	*
	*   // First row
	*   int iRow = query.addRow() ;
	*   query.setData( iRow, iCity, &quot;Minneapolis&quot; ) ;
	*   query.setData( iRow, iState, &quot;MN&quot; ) ;
	*   query.setData( iRow, iZip, &quot;55345&quot; ) ;
	*
	*   // Second row
	*   iRow = query.addRow() ;
	*   query.setData( iRow, iCity, &quot;St. Paul&quot; ) ;
	*   query.setData( iRow, iState, &quot;MN&quot; ) ;
	*   query.setData( iRow, iZip, &quot;55105&quot; ) ; </code></pre>
	*
	* @param iRow row of data element to set (1-based)
	* @param iCol column of data element to set (1-based)
	* @param data new value for data element
	*
	* @throws <code>IndexOutOfBoundsException</code> If an invalid index is passed to the method.
	*
   * @see #getData getData
	* @see #addRow addRow
	*/
	public void setData( int iRow, int iCol, String data ) throws IndexOutOfBoundsException ;


        /*adding a sort routine that needed to sort data results.
        * it takes tow arguments: column_index and sort_order.
        * column_index parameter to indicate which column to sort
        * sort_order parameter to indicate whether it is ASC or DESC
        */
        public void sort(int iCol, String sort_order);

        /*adding a sort routine that needed to sort data results.
        * it takes one arguments: the entire sort string
        * it will parse the string and sort the query object
        */
        public void sort(String sort_string);


}

